@@ -50,12 +50,14 @@ def usage():
     print >>sys.stderr, "  -r                           Process recursively"
     print >>sys.stderr, "  -E [extension]               Set extension (default: xml)"
     print >>sys.stderr, "  -O                           Output each file to similarly named .txt file"
+    print >>sys.stderr, "  -q                           Ignore errors"    
 
 class settings:
     output_header = True
     csv = False
     outputfile = None
     sentencespacing = True
+    ignoreerrors = False
     nicespacing = 0
     autooutput = False
     extension = 'xml'
@@ -65,7 +67,7 @@ class settings:
     
 def main():    
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "o:OhHSc:x:E:r", ["help", "csv"])
+        opts, args = getopt.getopt(sys.argv[1:], "o:OhHSc:x:E:rq", ["help", "csv"])
     except getopt.GetoptError, err:
         print str(err)
         usage()
@@ -95,6 +97,8 @@ def main():
             settings.extension = a
         elif o == '-r':
             settings.recurse = True
+        elif o == '-q':
+            settings.ignoreerrors = True            
         elif o == '--csv':
             settings.csv = True
         else:
@@ -134,136 +138,144 @@ def processdir(d, outputfile = None):
     print >>sys.stderr, "Searching in  " + d
     for f in glob.glob(d + '/*'):        
         if f[-len(settings.extension) - 1:] == '.' + settings.extension: 
-            process(f)
+            process(f, outputfile)
         elif settings.recurse and os.path.isdir(f):
-            processdir(f)
+            processdir(f, outputfile)
 
 def process(filename, outputfile=None):
-    print >>sys.stderr, "Processing " + filename
-    doc = folia.Document(file=filename)
-    prevsen = None
+    try:
+        print >>sys.stderr, "Processing " + filename
+        doc = folia.Document(file=filename)
+        prevsen = None
 
-    if settings.autooutput:    
-        if settings.csv:
-            ext = '.csv'
-        else: 
-            ext = '.columns'
-        if filename[-len(settings.extension) - 1:].lower() == '.' +settings.extension:
-            outfilename = filename[:-len(settings.extension) - 1] + ext
-        else:
-            outfilename += ext
-        
-        print >>sys.stderr, " Saving as " + outfilename
-        outputfile = codecs.open(outfilename,'w',settings.encoding)
-                
-        
-    if settings.nicespacing:
-        spacing = []
-        for c in settings.columnconf:
-            if c == 'n':
-                spacing.append(3)
-            elif c == 'N':
-                spacing.append(7)
-            elif c == 'poshead':
-                spacing.append(5)
+        if settings.autooutput:    
+            if settings.csv:
+                ext = '.csv'
+            else: 
+                ext = '.columns'
+            if filename[-len(settings.extension) - 1:].lower() == '.' +settings.extension:
+                outfilename = filename[:-len(settings.extension) - 1] + ext
             else:
-                spacing.append(settings.nicespacing)
-        
-    if settings.output_header:        
-        
-        if settings.csv:
-            columns = [ '"' + x.upper()  + '"' for x in settings.columnconf ]
-        else:
-            columns = [ x.upper()  for x in settings.columnconf ]
-
-        if settings.nicespacing and not settings.csv:
-            columns = [ resize(x, i, spacing) for i, x in enumerate(settings.columnconf) ]
-        
-        if settings.csv:
-            line = ','.join(columns)
-        else:
-            line = '\t'.join(columns)
-
-        if outputfile:
-            outputfile.write(line + '\n')
-        else:    
-            print line.encode(settings.encoding)    
-
-    wordnum = 0
-
-
-
-    for i, w in enumerate(doc.words()):
-        if w.sentence() != prevsen and i > 0:
-            if settings.sentencespacing:
-                if outputfile:
-                    outputfile.write('\n')
+                outfilename += ext
+            
+            print >>sys.stderr, " Saving as " + outfilename
+            outputfile = codecs.open(outfilename,'w',settings.encoding)
+                    
+            
+        if settings.nicespacing:
+            spacing = []
+            for c in settings.columnconf:
+                if c == 'n':
+                    spacing.append(3)
+                elif c == 'N':
+                    spacing.append(7)
+                elif c == 'poshead':
+                    spacing.append(5)
                 else:
-                    print         
-            wordnum = 0
-        prevsen = w.sentence()
-        wordnum += 1
-        columns = []
-        for c in settings.columnconf:
-            if c == 'id':
-                columns.append(w.id)
-            elif c == 'text':
-                columns.append(w.text())
-            elif c == 'n':
-                columns.append(str(wordnum))
-            elif c == 'N':
-                columns.append(str(i+1))
-            elif c == 'pos':
-                try:
-                    columns.append(w.annotation(folia.PosAnnotation).cls)    
-                except:
-                    columns.append('-')
-            elif c == 'poshead':
-                try:
-                    columns.append(w.annotation(folia.PosAnnotation).feat('head'))    
-                except:
-                    columns.append('-')     
-            elif c == 'lemma':
-                try:
-                    columns.append(w.annotation(folia.LemmaAnnotation).cls)    
-                except:
-                    columns.append('-')
-            elif c == 'sense':
-                try:
-                    columns.append(w.annotation(folia.SenseAnnotation).cls)    
-                except:
-                    columns.append('-')
-            elif c == 'phon':
-                try:
-                    columns.append(w.annotation(folia.PhonAnnotation).cls)    
-                except:
-                    columns.append('-')                
-            elif c == 'senid':
-                columns.append(w.sentence().id)
-            elif c == 'parid':            
-                try:
-                    columns.append(w.paragraph().id)
-                except:
-                    columns.append('-')
-            elif c:
-                print >>sys.stderr,"ERROR: Unsupported configuration: " + c
-                sys.exit(1)
+                    spacing.append(settings.nicespacing)
             
-        if settings.nicespacing and not settings.csv:
-            columns = [ resize(x,j, spacing) for j,x  in enumerate(columns) ]
+        if settings.output_header:        
             
-        if settings.csv:
-            line = ",".join([ '"' + x  + '"' for x in columns ]) 
-        else:
-            line = "\t".join(columns)
-            
-        if outputfile:
-            outputfile.write(line+'\n')
-        else:    
-            print line.encode(settings.encoding)
+            if settings.csv:
+                columns = [ '"' + x.upper()  + '"' for x in settings.columnconf ]
+            else:
+                columns = [ x.upper()  for x in settings.columnconf ]
 
-    if settings.autooutput:
-        outputfile.close()
+            if settings.nicespacing and not settings.csv:
+                columns = [ resize(x, i, spacing) for i, x in enumerate(settings.columnconf) ]
+            
+            if settings.csv:
+                line = ','.join(columns)
+            else:
+                line = '\t'.join(columns)
+
+            if outputfile:
+                outputfile.write(line + '\n')
+            else:    
+                print line.encode(settings.encoding)    
+
+        wordnum = 0
+
+
+
+        for i, w in enumerate(doc.words()):
+            if w.sentence() != prevsen and i > 0:
+                if settings.sentencespacing:
+                    if outputfile:
+                        outputfile.write('\n')
+                    else:
+                        print         
+                wordnum = 0
+            prevsen = w.sentence()
+            wordnum += 1
+            columns = []
+            for c in settings.columnconf:
+                if c == 'id':
+                    columns.append(w.id)
+                elif c == 'text':
+                    columns.append(w.text())
+                elif c == 'n':
+                    columns.append(str(wordnum))
+                elif c == 'N':
+                    columns.append(str(i+1))
+                elif c == 'pos':
+                    try:
+                        columns.append(w.annotation(folia.PosAnnotation).cls)    
+                    except:
+                        columns.append('-')
+                elif c == 'poshead':
+                    try:
+                        columns.append(w.annotation(folia.PosAnnotation).feat('head'))    
+                    except:
+                        columns.append('-')     
+                elif c == 'lemma':
+                    try:
+                        columns.append(w.annotation(folia.LemmaAnnotation).cls)    
+                    except:
+                        columns.append('-')
+                elif c == 'sense':
+                    try:
+                        columns.append(w.annotation(folia.SenseAnnotation).cls)    
+                    except:
+                        columns.append('-')
+                elif c == 'phon':
+                    try:
+                        columns.append(w.annotation(folia.PhonAnnotation).cls)    
+                    except:
+                        columns.append('-')                
+                elif c == 'senid':
+                    columns.append(w.sentence().id)
+                elif c == 'parid':            
+                    try:
+                        columns.append(w.paragraph().id)
+                    except:
+                        columns.append('-')
+                elif c:
+                    print >>sys.stderr,"ERROR: Unsupported configuration: " + c
+                    sys.exit(1)
+                
+            if settings.nicespacing and not settings.csv:
+                columns = [ resize(x,j, spacing) for j,x  in enumerate(columns) ]
+                
+            if settings.csv:
+                line = ",".join([ '"' + x  + '"' for x in columns ]) 
+            else:
+                line = "\t".join(columns)
+                
+            if outputfile:
+                outputfile.write(line+'\n')
+            else:    
+                print line.encode(settings.encoding)
+
+        if settings.autooutput:
+            outputfile.close()
+        elif outputfile:
+            outputfile.flush()
+    except Exception as e:
+        if settings.ignoreerrors:
+            print >>sys.stderr, "ERROR: An exception was raised whilst processing " + filename, e            
+        else:
+            raise        
         
 if __name__ == "__main__":
     main()
